import {create} from 'zustand'
import {Bucket, Connection} from "../interfaces/connection";
import {Dial, Resp} from "../api";

interface StoreBucket {
    bucket_list: Bucket[];
    bucket_get: (conn: Connection, refresh: boolean) => void;
}

let bucket_map: { [id: number]: Bucket[] };

export const useStoreBucket = create<StoreBucket>()((set) => ({
    bucket_list: [],
    bucket_get: async (conn: Connection, refresh: boolean) => {
        let res: Resp<{ list: Bucket[]; }>;
        if (refresh) {
            res = await Dial<{list: Bucket[]}>('/api/connection/buckets', {id: conn.id});
            if (res.status !== 200) {
                return
            }
        }

        set((state) => {
           if (refresh) {
               bucket_map = {...bucket_map, [conn.id]: res.data.list}
               return {bucket_list: res.data.list};
           }

           return {bucket_list: bucket_map[conn.id]};
        })
    }
}))
